@@ -48,7 +48,7 @@ typedef struct _EwsOabDecoderPrivate EwsOabDecoderPrivate;
 
 struct _EwsOabDecoderPrivate {
 	gchar *cache_dir;
-	GFileInputStream *fis;
+	GInputStream *fis;
 
 	guint32 total_records;
 	GSList *hdr_props;
@@ -62,6 +62,19 @@ typedef struct {
 	EContactAddress *addr;
 } EwsDeferredSet;
 
+static void
+ews_populate_string_sha1 (EContact *contact,
+			  EContactField field,
+			  gpointer value,
+			  gpointer user_data)
+{
+	gchar *sum;
+
+	sum = g_compute_checksum_for_string (G_CHECKSUM_SHA1, (const gchar *) value, -1);
+	e_contact_set (contact, field, sum);
+	g_free (sum);
+}
+
 static void
 ews_populate_simple_string (EContact *contact,
                             EContactField field,
@@ -207,6 +220,7 @@ static const struct prop_field_mapping {
 	void (*populate_function) (EContact *contact, EContactField field, gpointer value, gpointer user_data);
 	void (*defered_populate_function) (EwsDeferredSet *dset, guint32 prop_id, gpointer value);
 } prop_map[] = {
+	{EWS_PT_EMAIL_ADDRESS, E_CONTACT_UID, ews_populate_string_sha1},
 	{EWS_PT_SMTP_ADDRESS, E_CONTACT_EMAIL_1, ews_populate_simple_string},
 	{EWS_PT_DISPLAY_NAME, E_CONTACT_FULL_NAME, ews_populate_simple_string},
 	{EWS_PT_ACCOUNT, E_CONTACT_NICKNAME, ews_populate_simple_string},
@@ -307,7 +321,7 @@ ews_oab_decoder_new (const gchar *oab_filename,
 	priv = GET_PRIVATE (eod);
 
 	gf = g_file_new_for_path (oab_filename);
-	priv->fis = g_file_read (gf, NULL, &err);
+	priv->fis = (GInputStream *)g_file_read (gf, NULL, &err);
 	if (err)
 		goto exit;
 
@@ -443,16 +457,15 @@ typedef struct {
 } EwsOabHdr;
 
 static EwsOabHdr *
-ews_read_oab_header (EwsOabDecoder *eod,
+ews_read_oab_header (EwsOabDecoder *eod, GInputStream *stream,
                      GCancellable *cancellable,
                      GError **error)
 {
 	EwsOabHdr *o_hdr;
-	EwsOabDecoderPrivate *priv = GET_PRIVATE (eod);
 
 	o_hdr = g_new0 (EwsOabHdr, 1);
 
-	o_hdr->version = ews_oab_read_uint32 (G_INPUT_STREAM (priv->fis), cancellable, error);
+	o_hdr->version = ews_oab_read_uint32 (stream, cancellable, error);
 	if (*error)
 		goto exit;
 
@@ -461,10 +474,10 @@ ews_read_oab_header (EwsOabDecoder *eod,
 		goto exit;
 	}
 
-	o_hdr->serial = ews_oab_read_uint32 (G_INPUT_STREAM (priv->fis), cancellable, error);
+	o_hdr->serial = ews_oab_read_uint32 (stream, cancellable, error);
 	if (*error)
 		goto exit;
-	o_hdr->total_recs = ews_oab_read_uint32 (G_INPUT_STREAM (priv->fis), cancellable, error);
+	o_hdr->total_recs = ews_oab_read_uint32 (stream, cancellable, error);
 
 exit:
 	if (*error) {
@@ -476,7 +489,7 @@ exit:
 }
 
 static gboolean
-ews_decode_hdr_props (EwsOabDecoder *eod,
+ews_decode_hdr_props (EwsOabDecoder *eod, GInputStream *stream,
                       gboolean oab_hdrs,
                       GCancellable *cancellable,
                       GError **error)
@@ -486,9 +499,7 @@ ews_decode_hdr_props (EwsOabDecoder *eod,
 	GSList **props;
 
 	/* number of properties */
-	num_props = ews_oab_read_uint32 (
-		G_INPUT_STREAM (priv->fis),
-		cancellable, error);
+	num_props = ews_oab_read_uint32 (stream, cancellable, error);
 
 	if (*error)
 		return FALSE;
@@ -506,9 +517,7 @@ ews_decode_hdr_props (EwsOabDecoder *eod,
 	for (i = 0; i < num_props; i++) {
 		guint32 prop_id;
 
-		prop_id = ews_oab_read_uint32 (
-			G_INPUT_STREAM (priv->fis),
-			cancellable, error);
+		prop_id = ews_oab_read_uint32 (stream, cancellable, error);
 
 		*props = g_slist_prepend (*props, GUINT_TO_POINTER (prop_id));
 
@@ -516,9 +525,7 @@ ews_decode_hdr_props (EwsOabDecoder *eod,
 			return FALSE;
 
 		/* eat the flags */
-		ews_oab_read_uint32 (
-			G_INPUT_STREAM (priv->fis),
-			cancellable, error);
+		ews_oab_read_uint32 (stream, cancellable, error);
 
 		if (*error)
 			return FALSE;
@@ -533,26 +540,23 @@ ews_decode_hdr_props (EwsOabDecoder *eod,
 }
 
 static gboolean
-ews_decode_metadata (EwsOabDecoder *eod,
+ews_decode_metadata (EwsOabDecoder *eod, GInputStream *stream,
                      GCancellable *cancellable,
                      GError **error)
 {
-	EwsOabDecoderPrivate *priv = GET_PRIVATE (eod);
 	gboolean ret = TRUE;
 
 	/* eat the size */
-	ews_oab_read_uint32 (
-		G_INPUT_STREAM (priv->fis),
-		cancellable, error);
+	ews_oab_read_uint32 (stream, cancellable, error);
 
 	if (*error)
 		return FALSE;
 
-	ret = ews_decode_hdr_props (eod, FALSE, cancellable, error);
+	ret = ews_decode_hdr_props (eod, stream, FALSE, cancellable, error);
 	if (!ret)
 		return FALSE;
 
-	ret = ews_decode_hdr_props (eod, TRUE, cancellable, error);
+	ret = ews_decode_hdr_props (eod, stream, TRUE, cancellable, error);
 
 	return ret;
 }
@@ -573,15 +577,14 @@ ews_is_bit_set (const gchar *str,
 }
 
 static guint32
-ews_decode_uint32 (EwsOabDecoder *eod,
+ews_decode_uint32 (EwsOabDecoder *eod, GInputStream *stream,
                    GCancellable *cancellable,
                    GError **error)
 {
-	EwsOabDecoderPrivate *priv = GET_PRIVATE (eod);
 	guint8 first;
 	guint32 ret = 0, num;
 
-	g_input_stream_read (G_INPUT_STREAM (priv->fis), &first, 1, cancellable, error);
+	g_input_stream_read (stream, &first, 1, cancellable, error);
 	if (*error)
 		return ret;
 
@@ -591,16 +594,16 @@ ews_decode_uint32 (EwsOabDecoder *eod,
 		return (guint32) first;
 
 	if (num == 1) {
-		g_input_stream_read (G_INPUT_STREAM (priv->fis), &first, 1, cancellable, error);
+		g_input_stream_read (stream, &first, 1, cancellable, error);
 		return (guint32) first;
 	}
 
 	if (num == 2)
-		ret = ews_oab_read_uint16 (G_INPUT_STREAM (priv->fis), cancellable, error);
+		ret = ews_oab_read_uint16 (stream, cancellable, error);
 	if (num == 3) {
 		gchar *tmp, *str = g_malloc0 (num + 1);
 
-		g_input_stream_read (G_INPUT_STREAM (priv->fis), str, num, cancellable, error);
+		g_input_stream_read (stream, str, num, cancellable, error);
 		/* not sure if its the right way to do, test it */
 		tmp = g_strconcat ("0", str, NULL);
 
@@ -610,26 +613,25 @@ ews_decode_uint32 (EwsOabDecoder *eod,
 		g_free (str);
 		g_free (tmp);
 	} else if (num == 4)
-		ret = ews_oab_read_uint32 (G_INPUT_STREAM (priv->fis), cancellable, error);
+		ret = ews_oab_read_uint32 (stream, cancellable, error);
 
 	return ret;
 }
 
 static GBytes *
-ews_decode_binary (EwsOabDecoder *eod,
+ews_decode_binary (EwsOabDecoder *eod, GInputStream *stream,
                    GCancellable *cancellable,
                    GError **error)
 {
-	EwsOabDecoderPrivate *priv = GET_PRIVATE (eod);
 	guint32 len;
 	gchar *binary;
 	GBytes *val = NULL;
 
-	len = ews_decode_uint32 (eod, cancellable, error);
+	len = ews_decode_uint32 (eod, stream, cancellable, error);
 	if (*error)
 		return NULL;
 	binary = g_malloc (len);
-	g_input_stream_read (G_INPUT_STREAM (priv->fis), binary, len, cancellable, error);
+	g_input_stream_read (stream, binary, len, cancellable, error);
 	if (*error) {
 		g_free (binary);
 		goto exit;
@@ -643,12 +645,11 @@ exit:
 }
 
 static gpointer
-ews_decode_oab_prop (EwsOabDecoder *eod,
+ews_decode_oab_prop (EwsOabDecoder *eod, GInputStream *stream,
                      guint32 prop_id,
                      GCancellable *cancellable,
                      GError **error)
 {
-	EwsOabDecoderPrivate *priv = GET_PRIVATE (eod);
 	guint32 prop_type;
 	gpointer ret_val = NULL;
 
@@ -659,7 +660,7 @@ ews_decode_oab_prop (EwsOabDecoder *eod,
 		{
 			guint32 val;
 
-			val = ews_decode_uint32 (eod, cancellable, error);
+			val = ews_decode_uint32 (eod, stream, cancellable, error);
 			ret_val = GUINT_TO_POINTER (val);
 
 			d (g_print ("prop id %X prop type: int32 value %d \n", prop_id, val);)
@@ -670,7 +671,7 @@ ews_decode_oab_prop (EwsOabDecoder *eod,
 		{
 			guchar val;
 
-			g_input_stream_read (G_INPUT_STREAM (priv->fis), &val, 1, cancellable, error);
+			g_input_stream_read (stream, &val, 1, cancellable, error);
 			ret_val = GUINT_TO_POINTER ((guint) val);
 			d (g_print ("prop id %X prop type: bool value %d \n", prop_id, val);)
 
@@ -681,7 +682,7 @@ ews_decode_oab_prop (EwsOabDecoder *eod,
 		{
 			gchar *val;
 
-			val = ews_oab_read_upto (G_INPUT_STREAM (priv->fis), '\0', cancellable, error);
+			val = ews_oab_read_upto (stream, '\0', cancellable, error);
 			ret_val = (gpointer) val;
 
 			d (g_print ("prop id %X prop type: string value %s \n", prop_id, val);)
@@ -689,7 +690,7 @@ ews_decode_oab_prop (EwsOabDecoder *eod,
 		}
 		case EWS_PTYP_BINARY:
 		{
-			ret_val = ews_decode_binary (eod, cancellable, error);
+			ret_val = ews_decode_binary (eod, stream, cancellable, error);
 			d (g_print ("prop id %X prop type: binary size %zd \n", prop_id, g_bytes_get_size ((GBytes *)ret_val)));
 			break;
 		}
@@ -701,7 +702,7 @@ ews_decode_oab_prop (EwsOabDecoder *eod,
 			guint32 num, i;
 			GSList *list = NULL;
 
-			num = ews_decode_uint32 (eod, cancellable, error);
+			num = ews_decode_uint32 (eod, stream, cancellable, error);
 			if (*error)
 				break;
 			d (g_print ("prop id %X prop type: multi-num %d \n", prop_id, num);)
@@ -712,7 +713,7 @@ ews_decode_oab_prop (EwsOabDecoder *eod,
 				if (prop_type == EWS_PTYP_MULTIPLEINTEGER32) {
 					guint32 v = 0;
 
-					v = ews_decode_uint32 (eod, cancellable, error);
+					v = ews_decode_uint32 (eod, stream, cancellable, error);
 					val = GUINT_TO_POINTER (v);
 					list = g_slist_prepend (list, val);
 
@@ -724,7 +725,7 @@ ews_decode_oab_prop (EwsOabDecoder *eod,
 				} else if (prop_type == EWS_PTYP_MULTIPLEBINARY) {
 					GBytes *val;
 
-					val = ews_decode_binary (eod, cancellable, error);
+					val = ews_decode_binary (eod, stream, cancellable, error);
 					if (!val) {
 						g_slist_foreach (list, (GFunc) g_bytes_unref, NULL);
 						g_slist_free (list);
@@ -737,7 +738,7 @@ ews_decode_oab_prop (EwsOabDecoder *eod,
 				} else {
 					gchar *val;
 
-					val = ews_oab_read_upto (G_INPUT_STREAM (priv->fis), '\0', cancellable, error);
+					val = ews_oab_read_upto (stream, '\0', cancellable, error);
 					if (!val) {
 						g_slist_foreach (list, (GFunc) g_free, NULL);
 						g_slist_free (list);
@@ -876,7 +877,6 @@ ews_decode_addressbook_write_display_type (EContact **contact,
  * @eod: 
  * @contact: Pass a valid EContact for decoding the address-book record. NULL in case of header record.
  * @props:
- * @dset: used to collect multiple properties that needs to be combined and stored as one EContactField
  * @cancellable: 
  * @error: 
  * 
@@ -886,14 +886,14 @@ ews_decode_addressbook_write_display_type (EContact **contact,
  * Returns: 
  **/
 static gboolean
-ews_decode_addressbook_record (EwsOabDecoder *eod,
+ews_decode_addressbook_record (EwsOabDecoder *eod, GInputStream *stream,
                                EContact *contact,
-                               EwsDeferredSet *dset,
                                GSList *props,
                                GCancellable *cancellable,
                                GError **error)
 {
 	EwsOabDecoderPrivate *priv = GET_PRIVATE (eod);
+	EwsDeferredSet *dset = NULL;
 	guint bit_array_size, i, len;
 	gchar *bit_str;
 	gboolean ret = TRUE;
@@ -901,12 +901,15 @@ ews_decode_addressbook_record (EwsOabDecoder *eod,
 	len = g_slist_length (props);
 	bit_array_size = (guint) ceil (len / 8.0);
 	bit_str = g_malloc0 (bit_array_size);
-	g_input_stream_read (G_INPUT_STREAM (priv->fis), bit_str, bit_array_size, cancellable, error);
+	g_input_stream_read (stream, bit_str, bit_array_size, cancellable, error);
 	if (*error) {
 		ret = FALSE;
 		goto exit;
 	}
 
+	if (contact)
+		dset = g_new0 (EwsDeferredSet, 1);
+
 	for (i = 0; i < len; i++) {
 		gpointer val, index;
 		guint32 prop_id;
@@ -923,7 +926,7 @@ ews_decode_addressbook_record (EwsOabDecoder *eod,
 		if ((prop_id & 0xFFFF) == EWS_PTYP_OBJECT)
 			continue;
 
-		val = ews_decode_oab_prop (eod, prop_id, cancellable, error);
+		val = ews_decode_oab_prop (eod, stream, prop_id, cancellable, error);
 
 		if (prop_id == EWS_PT_DISPLAY_TYPE)
 			ews_decode_addressbook_write_display_type (&contact, GPOINTER_TO_UINT (val), FALSE);
@@ -950,88 +953,111 @@ exit:
 	if (bit_str)
 		g_free (bit_str);
 
+	if (!contact)
+		return ret;
+
+	if (dset->addr) {
+		e_contact_set (contact, E_CONTACT_ADDRESS_WORK, dset->addr);
+		e_contact_address_free (dset->addr);
+	}
+	g_free (dset);
+
+	/* set the smtp address as contact's uid */
+	if (!e_contact_get_const(contact, E_CONTACT_UID)) {
+		const gchar *uid = e_contact_get_const (contact, E_CONTACT_EMAIL_1);
+		if (uid && *uid)
+			e_contact_set (contact, E_CONTACT_UID, uid);
+		else
+			ret = FALSE;
+	}
+
 	return ret;
 }
 
 /* Decodes the hdr and address-book records and stores the address-book records inside the db */
 static gboolean
 ews_decode_and_store_oab_records (EwsOabDecoder *eod,
+				  EwsOabContactFilterCb filter_cb,
                                   EwsOabContactAddedCb cb,
                                   gpointer user_data,
                                   GCancellable *cancellable,
                                   GError **error)
 {
 	EwsOabDecoderPrivate *priv = GET_PRIVATE (eod);
-	gboolean ret = TRUE;
+	gboolean ret = FALSE;
 	guint32 i;
+	int buf_len = 200;
+	guchar *record_buf = g_malloc (buf_len);
+	GChecksum *sum = g_checksum_new (G_CHECKSUM_SHA1);
+
+	if (!record_buf || !sum)
+		goto exit;
 
 	/* eat the size */
 	ews_oab_read_uint32 (
-		G_INPUT_STREAM (priv->fis),
+		priv->fis,
 		cancellable, error);
 
-	ews_decode_addressbook_record (
-		eod, NULL, NULL, priv->hdr_props, cancellable, error);
+	ews_decode_addressbook_record (eod, priv->fis, NULL,
+				       priv->hdr_props, cancellable, error);
 
-	if (*error) {
-		ret = FALSE;
+	if (*error)
 		goto exit;
-	}
+
 
 	for (i = 0; i < priv->total_records; i++) {
 		EContact *contact;
-		EwsDeferredSet *dset;
-		gchar *uid = NULL;
 		goffset offset;
+		guint32 rec_size;
+		GInputStream *memstream;
+		const gchar *sum_str;
 
 		contact = e_contact_new ();
-		dset = g_new0 (EwsDeferredSet, 1);
 
 		/* eat the size */
-		ews_oab_read_uint32 (
-			G_INPUT_STREAM (priv->fis),
-			cancellable, error);
+		rec_size = ews_oab_read_uint32 (priv->fis, cancellable, error);
+		if (rec_size < 4)
+			goto exit;
 
+		rec_size -= 4;
+
+		if (rec_size > buf_len) {
+			g_free (record_buf);
+			record_buf = g_malloc(rec_size);
+			buf_len = rec_size;
+			if (!record_buf)
+				goto exit;
+		}
 		/* fetch the offset */
 		offset = g_seekable_tell ((GSeekable *) priv->fis);
+		if (g_input_stream_read (priv->fis, record_buf, rec_size, cancellable, error) != rec_size)
+			goto exit;
 
-		ews_decode_addressbook_record (
-			eod, contact, dset,
-			priv->oab_props, cancellable, error);
+		g_checksum_reset (sum);
+		g_checksum_update (sum, record_buf, rec_size);
+		sum_str = g_checksum_get_string (sum);
+
+		memstream = g_memory_input_stream_new_from_data (record_buf, rec_size, NULL);
+
+		if ((!filter_cb || filter_cb (offset, sum_str, user_data, error)) &&
+		    ews_decode_addressbook_record (eod, memstream,
+						   contact, priv->oab_props,
+						   cancellable, error))
+			cb (contact, offset, sum_str,
+			    ((gfloat) (i + 1) / priv->total_records) * 100,
+			    user_data, error);
+
+		g_object_unref (memstream);
+		g_object_unref (contact);
 
 		if (*error)
-			goto error;
-
-		if (dset->addr)
-			e_contact_set (
-				contact,
-				E_CONTACT_ADDRESS_WORK,
-				dset->addr);
-
-		/* set the smtp address as contact's uid */
-		uid = (gchar *) e_contact_get (contact, E_CONTACT_EMAIL_1);
-		if (uid && *uid) {
-			e_contact_set (contact, E_CONTACT_UID, uid);
-
-			cb (
-				contact, offset,
-				((gfloat) (i + 1) / priv->total_records) * 100,
-				user_data, error);
-		}
-
-error:
-		g_object_unref (contact);
-		e_contact_address_free (dset->addr);
-		g_free (dset);
-		g_free (uid);
-
-		if (*error) {
-			ret = FALSE;
 			goto exit;
-		}
 	}
 
+	ret = TRUE;
 exit:
+	g_checksum_free (sum);
+	g_free (record_buf);
 	return ret;
 }
 
@@ -1118,6 +1144,7 @@ ews_oab_decoder_set_oab_prop_string (EwsOabDecoder *eod,
  **/
 gboolean
 ews_oab_decoder_decode (EwsOabDecoder *eod,
+                        EwsOabContactFilterCb filter_cb,
                         EwsOabContactAddedCb cb,
                         gpointer user_data,
                         GCancellable *cancellable,
@@ -1128,7 +1155,7 @@ ews_oab_decoder_decode (EwsOabDecoder *eod,
 	EwsOabHdr *o_hdr;
 	gboolean ret = TRUE;
 
-	o_hdr = ews_read_oab_header (eod, cancellable, &err);
+	o_hdr = ews_read_oab_header (eod, priv->fis, cancellable, &err);
 	if (!o_hdr) {
 		ret = FALSE;
 		goto exit;
@@ -1137,12 +1164,12 @@ ews_oab_decoder_decode (EwsOabDecoder *eod,
 	priv->total_records = o_hdr->total_recs;
 	g_print ("Total records is %d \n", priv->total_records);
 
-	ret = ews_decode_metadata (eod, cancellable, &err);
+	ret = ews_decode_metadata (eod, priv->fis, cancellable, &err);
 	if (!ret)
 		goto exit;
 
 	ret = ews_decode_and_store_oab_records (
-		eod, cb, user_data, cancellable, &err);
+		eod, filter_cb, cb, user_data, cancellable, &err);
 exit:
 	if (o_hdr)
 		g_free (o_hdr);
@@ -1161,23 +1188,19 @@ ews_oab_decoder_get_contact_from_offset (EwsOabDecoder *eod,
                                          GError **error)
 {
 	EwsOabDecoderPrivate *priv = GET_PRIVATE (eod);
-	EwsDeferredSet *dset;
 	EContact *contact = NULL;
 
 	if (!g_seekable_seek ((GSeekable *) priv->fis, offset, G_SEEK_SET, cancellable, error))
 		return NULL;
 
 	contact = e_contact_new ();
-	dset = g_new0 (EwsDeferredSet, 1);
-	ews_decode_addressbook_record (eod, contact, dset, oab_props, cancellable, error);
-	if (*error) {
+	if (!ews_decode_addressbook_record (eod, priv->fis,
+					    contact, oab_props, cancellable,
+					    error)) {
 		g_object_unref (contact);
 		contact = NULL;
 	}
 
-	e_contact_address_free (dset->addr);
-	g_free (dset);
-
 	return contact;
 }
 
