@@ -123,13 +123,19 @@ e_cal_backend_ews_populate_windows_zones (void)
 		tokens = g_strsplit ((gchar *) ical, " ", 0);
 		tokens_len = g_strv_length (tokens);
 		if (tokens_len == 1) {
-			g_hash_table_insert (msdn_to_ical, g_strdup ((gchar *) msdn), g_strdup ((gchar *) ical));
-			g_hash_table_insert (ical_to_msdn, g_strdup ((gchar *) ical), g_strdup ((gchar *) msdn));
+			if (!g_hash_table_lookup (msdn_to_ical, msdn))
+				g_hash_table_insert (msdn_to_ical, g_strdup ((gchar *) msdn), g_strdup ((gchar *) ical));
+
+			if (!g_hash_table_lookup (ical_to_msdn, ical))
+				g_hash_table_insert (ical_to_msdn, g_strdup ((gchar *) ical), g_strdup ((gchar *) msdn));
 		} else {
 			gint j;
 			for (j = 0; j < tokens_len; j++) {
-				g_hash_table_insert (msdn_to_ical, g_strdup ((gchar *) msdn), g_strdup (tokens[j]));
-				g_hash_table_insert (ical_to_msdn, g_strdup (tokens[j]), g_strdup ((gchar *) msdn));
+				if (!g_hash_table_lookup (msdn_to_ical, msdn))
+					g_hash_table_insert (msdn_to_ical, g_strdup ((gchar *) msdn), g_strdup (tokens[j]));
+
+				if (!g_hash_table_lookup (ical_to_msdn, tokens[j]))
+					g_hash_table_insert (ical_to_msdn, g_strdup (tokens[j]), g_strdup ((gchar *) msdn));
 			}
 		}
 
@@ -210,37 +216,6 @@ e_cal_backend_ews_tz_util_get_ical_equivalent (const gchar *msdn_tz_location)
 	return ical_tz_location;
 }
 
-EwsCalendarConvertData *
-ews_calendar_convert_data_new (void)
-{
-	return g_new0 (EwsCalendarConvertData, 1);
-}
-
-void
-ews_calendar_convert_data_free (EwsCalendarConvertData *convert_data)
-{
-	if (convert_data != NULL) {
-		if (convert_data->connection != NULL)
-			g_clear_object (&convert_data->connection);
-		if (convert_data->comp != NULL)
-			g_clear_object (&convert_data->comp);
-		if (convert_data->old_comp != NULL)
-			g_clear_object (&convert_data->old_comp);
-		if (convert_data->default_zone != NULL)
-			icaltimezone_free (convert_data->default_zone, TRUE);
-		if (convert_data->icalcomp != NULL)
-			icalcomponent_free (convert_data->icalcomp);
-
-		g_free (convert_data->item_id);
-		g_free (convert_data->change_key);
-		g_free (convert_data->user_email);
-		g_free (convert_data->response_type);
-		g_slist_free_full (convert_data->users, g_free);
-
-		g_free (convert_data);
-	}
-}
-
 /*
  * Iterate over the icalcomponent properties and collect attendees
  */
@@ -1303,7 +1278,7 @@ convert_vevent_calcomp_to_xml (ESoapMessage *msg,
 	ical_location_end = icaltimezone_get_location (tzid_end);
 
 	satisfies = e_ews_connection_satisfies_server_version (convert_data->connection, E_EWS_EXCHANGE_2010);
-	if (satisfies) {
+	if (satisfies && ical_location_start != NULL && ical_location_end != NULL) {
 		/* set iana timezone info as an extended property */
 		e_ews_message_add_extended_property_distinguished_name_string (
 			msg,
@@ -1664,7 +1639,7 @@ convert_vevent_component_to_updatexml (ESoapMessage *msg,
 	}
 
 	org_email_address = e_ews_collect_organizer (icalcomp);
-	if (org_email_address && g_ascii_strcasecmp (org_email_address, convert_data->user_email)) {
+	if (org_email_address && convert_data->user_email && g_ascii_strcasecmp (org_email_address, convert_data->user_email)) {
 		e_ews_message_end_item_change (msg);
 		return;
 	}
@@ -1842,7 +1817,7 @@ convert_vevent_component_to_updatexml (ESoapMessage *msg,
 			e_ews_message_replace_server_version (msg, E_EWS_EXCHANGE_2007_SP1);
 
 			e_ews_message_start_set_item_field (msg, "MeetingTimeZone", "calendar", "CalendarItem");
-			ewscal_set_meeting_timezone (msg, tzid_start);
+			ewscal_set_meeting_timezone (msg, tzid_start ? tzid_start : convert_data->default_zone);
 			e_ews_message_end_set_item_field (msg);
 		}
 	}
