@@ -36,6 +36,7 @@
 #include <shell/e-shell-window.h>
 
 #include "server/e-ews-connection.h"
+#include "server/e-ews-connection-utils.h"
 #include "server/e-source-ews-folder.h"
 
 #include "e-ews-edit-folder-permissions.h"
@@ -264,6 +265,13 @@ struct _EEwsConfigUtilsAuthenticatorClass {
 	GObjectClass parent_class;
 };
 
+static gboolean
+ews_config_utils_authenticator_get_without_password (ESourceAuthenticator *auth)
+{
+	EEwsConfigUtilsAuthenticator *authenticator = (EEwsConfigUtilsAuthenticator *) auth;
+	return e_ews_connection_utils_get_without_password (authenticator->ews_settings);
+}
+
 static ESourceAuthenticationResult
 ews_config_utils_authenticator_try_password_sync (ESourceAuthenticator *auth,
                                                   const GString *password,
@@ -328,7 +336,7 @@ ews_config_utils_authenticator_try_password_sync (ESourceAuthenticator *auth,
 
 GType e_ews_config_utils_authenticator_get_type (void) G_GNUC_CONST;
 
-static void e_ews_config_utils_authenticator_authenticator_init (ESourceAuthenticatorInterface *interface);
+static void e_ews_config_utils_authenticator_authenticator_init (ESourceAuthenticatorInterface *iface);
 
 G_DEFINE_TYPE_EXTENDED (EEwsConfigUtilsAuthenticator, e_ews_config_utils_authenticator, G_TYPE_OBJECT, 0,
 	G_IMPLEMENT_INTERFACE (E_TYPE_SOURCE_AUTHENTICATOR, e_ews_config_utils_authenticator_authenticator_init))
@@ -356,9 +364,12 @@ e_ews_config_utils_authenticator_class_init (EEwsConfigUtilsAuthenticatorClass *
 }
 
 static void
-e_ews_config_utils_authenticator_authenticator_init (ESourceAuthenticatorInterface *interface)
+e_ews_config_utils_authenticator_authenticator_init (ESourceAuthenticatorInterface *iface)
 {
-	interface->try_password_sync = ews_config_utils_authenticator_try_password_sync;
+	iface->get_without_password =
+		ews_config_utils_authenticator_get_without_password;
+	iface->try_password_sync =
+		ews_config_utils_authenticator_try_password_sync;
 }
 
 static void
